/*
 *
 *  Bluemodem - Bluetooth modem configuration and connection program
 *
 *  Copyright (C) 2006-2008  Sean Godsell <sgodsell@hotmail.com>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>


#include "bluemodem.h"

int removeConfiguration(char *cfgname);
int getConfiguration(char *cfgname);
int listConfigurations();
int getDeviceNumber();
int getDeviceFromFile(char *cfgname);
int removeUpdatedResolv();
int updateResolv();
int checkToUpdateResolv(char *cfgname);
int checkPPPResolv();
int checkPPPD(char *cfgname);
int runPPPD(char *cfgname, int verbose);
int getChannel();
int dropConnection(char *cfgname);
int saveFiles(char *cfgname);
int promptSettings();
int testBluetoothSetup(char *cfgname);
int lookupName(FILE *fs, int display, char *findstr, char *baddr, char *dname);
int getDevices(int display, char *findstr);
void dropSpecialChars(char *data);

//char *dirppp = "/etc/ppp/";
char *dirppppeers = "/etc/ppp/peers/";
char *sysconfigname = "/etc/ppp/bluemodem.conf";
char *deffilename = "bluemodem";
char *filesysresolv= "/etc/resolv.conf";
char *oldfilepppresolv= "/etc/ppp/resolv.conf";
char *newfilepppresolv="/var/run/ppp/resolv.conf";
char *bluetoothdir = "/var/lib/bluetooth";
char *logfile= "/var/log/messages";

char baddr[8192];
char dname[8192];

int  verbose = 0;
int  nscnt = 0;
char nameservers[4][50];

char phonenum[60] = "*99***2#";
int  timeout = 25;
int  baudrate = 115200;
int  channel = 2;
int  devicenum = 0;
char commdevice[64] = "/dev/rfcomm0";
char configname[4096];

char *phonenumstr = "$$PHONENUM$$";
char *timeoutstr = "$$TIMEOUT$$";
char *baudratestr = "$$BAUDRATE$$";
char *commdevicestr = "$$DEVICE$$";
char *chatcmdstr = "$$CHATCMD$$";
char *filebluemodemstr = "$$BLUEMODEM$$";


char *bluemodem = "TIMEOUT         5\nECHO            ON\nABORT           '\\nBUSY\\r'\nABORT           '\\nERROR\\r'\nABORT           '\\nNO ANSWER\\r'\nABORT           '\\nNO CARRIER\\r'\nABORT           '\\nNO DIALTONE\\r'\nABORT           '\\nRINGING\\r\\n\\rRINGING\\r'\n''              \\rAT\nTIMEOUT         12\nOK              ATD$$PHONENUM$$\nTIMEOUT         $$TIMEOUT$$\nCONNECT         \"\"\n";

char *peerblue = "debug\nnoauth\nusepeerdns\n$$DEVICE$$\n$$BAUDRATE$$\nlocal\nnocrtscts\ndefaultroute\nnoipdefault\nconnect '$$CHATCMD$$ -f $$BLUEMODEM$$ -r /dev/console'\n";

void usage() {
    printf("bluemodem - %s\nby Sean Godsell\n", BMVERSION);
    printf("Usage:  bluemodem [-options] <cfg name>\n");
    printf("  Options\n");
    printf("\t-v\t\tversion\n");
    printf("\t-l\t\tlist all the configurations\n");
    printf("\t-s\t\tshow a list of bluetooth devices\n");
    printf("\t-r <cfg name>\tremove a specified configuration\n");
    printf("\t-d <cfg name>\tdrop a currently active modem connection\n");
    printf("\t-c\t\tconfigure the bluetooth modem settings (interactive)\n");
    printf("\t-c 1 2 3[4 5 6]\tconfigure none interactivly\n");
    printf("\t\t\tThe first (1) parameter is the configuration name\n");
    printf("\t\t\tThe 2 parameter is the device name or address\n");
    printf("\t\t\tThe 3 parameter is the phone # (access #)\n");
    printf("\t\t\tThe 4 parameter is the timeout value in seconds\n");
    printf("\t\t\tThe 5 parameter is the baudrate (default 115200)\n");
    printf("\t\t\tThe 6 parameter is the device (default /dev/rfcomm0)\n");
    printf("  To connect to a bluetooth modem (cell phone) type in the following:\n");
    printf("\tbluemodem <cfg name>\n");
    printf("\n");
}


/*
 * Start the following items:
 *	modules hci_uart, hci_usb
 *	make device rfcomm
 *	start the bluetoothd daemon
 */
int startBluetooth() {

    FILE *fs;
    char buf[16384];
    int  foundbluetoothd = 0, sock, dev_id, pid;

    dev_id = hci_get_route(NULL);
    sock = hci_open_dev( dev_id );
    if (dev_id < 0 || sock < 0) {
        /* Load the bluetooth kernel modules
         */
        sprintf(buf, "%s hci_uart 2> /dev/null > /dev/null", modprobecmd);
        system(buf);
        sprintf(buf, "%s hci_usb 2> /dev/null > /dev/null", modprobecmd);
        system(buf);
#ifdef ADD_MKNOD
        sprintf(buf, "%s %s c 216 0 2> /dev/null > /dev/null", mknodcmd, commdevice);
        system(buf);
#endif

        /* Start the bluetoothd daemon if it is not started
         */
        sprintf(buf, "%s 2> /dev/null > /dev/null", bluetoothdcmd);
        system(buf);
        sleep(2);
    }
    close(sock);

    /* Submit the hcitool to scan */
    /* This fixes a problem with the api scan */
    
    pid = fork();
    if (pid == 0) {
        sprintf(buf, "%s scan 2> /dev/null > /dev/null", hcitool);
        system(buf);
        exit(0);
    }

}

int main(int argc, char *argv[], char *env[]) {

    FILE *fs;
    char buf[16384];
    int  found = 0;
    long pid;
    int  chgconfig = 0;
    int  rtc;
    int  c = 0,i;
    char *parms[5];
    char pm[5][100];

    /* Start hcid daemon and kernel modules (if needed)
     */
    startBluetooth();

    while (c != -1)
    {
        c = getopt (argc, argv, "r:csd:vlh");

        switch (c)
        {
            /* List all the configurations */
            case 'v':
                printf("bluemodem - version %s\n", BMVERSION);
                exit(0);
                break;

            /* List all the configurations */
            case 'l':
                listConfigurations();
                exit(0);
                break;

            /* Remove a configuration */
            case 'r':
                if (optarg == NULL) {
                    printf("You must pass a configuration name\n");
                    exit(1);
                }
                if (!getConfiguration(optarg)) {
                    printf("The configuration name was not found!\n");
                    exit(1);
                }
                if (removeConfiguration(optarg))
                    printf("The %s configuration has been removed\n", optarg);
                else
                    printf("Error cannot remove the configuration (%s)!\n", optarg);
                exit(0);
                break;

            /* Drop a connection */
            case 'd':
                if (optarg == NULL) {
                    printf("You must pass a configuration name\n");
                    exit(1);
                }
                removeUpdatedResolv(optarg);
                if (!dropConnection(optarg))
                    printf("There was no connection available!\n");
                exit(0);
                break;

            /* Setup a configuration */
            case 'c':
                chgconfig = 1;
                break;

            /* Show a list of devices */
            case 's':
                getDevices(1, NULL);
                exit(0);
                break;

            /* Show the command usage or help */
            case '?':
            case 'h':
            case -1:
                if (c == -1 && argc > 1)
                break;
                usage();
                exit(0);
                break;
        }
    }

    strcpy(configname, deffilename);

    if (chgconfig && optind < argc) {
        strcpy(buf, argv[optind++]);
        found = getConfiguration(buf);
        if (found) {
            printf("The configuration name already exists! (%s)\n", buf);
            exit(1);
        }
        strncpy(configname, buf, 25);
        configname[25] = 0;
        dropSpecialChars(configname);
        if (optind < argc) {
            strcpy(buf, argv[optind++]);
            dropSpecialChars(buf);
            found = getDevices(0, buf);
            if (!found) {
                printf("The device name or address cannot be found! (%s)\n", buf);
                exit(1);
            }
        }

        if (optind < argc) {
            strncpy(phonenum, argv[optind++], 20);
            phonenum[20] = 0;
            dropSpecialChars(phonenum);
        } else {
            printf("You must have at least 3 parameters after the c option\n");
            printf("  <cfg name> <devicename/address> <phone #/access #>\n");
            exit(1);
        }

        if (optind < argc)
            timeout = atol(argv[optind++]);
        if (optind < argc)
            baudrate = atol(argv[optind++]);
        if (optind < argc) {
            strncpy(commdevice, argv[optind++], 60);
            commdevice[60] = 0;
            dropSpecialChars(commdevice);
        }
        if (saveFiles(configname))
            printf("\nError cannot save the configuration!\n");
        else
            printf("\nThe bluetooth modem settings are now saved!  Run bluemodem again.\n");
        exit(0);
    }

    if (!chgconfig) {
        strcpy(configname, argv[optind++]);
        dropSpecialChars(configname);
    }

    /* Check to see if the configuration files have been setup
     * or a user wants to change the configuration
     */
    if ( testBluetoothSetup(configname) || chgconfig) {
        if (!chgconfig) {
            printf("\nThe requested configuration does not exist (%s)\n", configname);
            exit(0);
        }
        if (promptSettings())
            printf("\nError cannot save the configuration!\n");
        else
            printf("\nThe configuration files are now saved!  Run bluemodem again.\n");
        exit(0);
    }


    /* Get the bluetooth device address and name
     */
    getConfiguration(configname);

    /* Get the channel to the device
     */
    channel = getChannel();
    if (channel == -1) {
        printf("The dialup networking cannot be found on the device!\n");
        exit(0);
    }

    /* Start the bluetooth modem connection with a phone
     *
     * Clean up the old resolv.conf file
     */
    unlink(oldfilepppresolv);
    unlink(newfilepppresolv);

    /* Get the device number for the communication device
     */
    getDeviceFromFile(configname);
    devicenum = getDeviceNumber();
    if (devicenum < 0) {
        printf("Error cannot determine the device number for %s\n", rfcommcmd);
        exit(0);
    }

    pid = fork();
    if (pid == -1) {
        printf("Error creating the child process. (could be a memory issue)\n");
        exit(0);
    }
    if (pid == 0) {
        sprintf(buf, "%s connect %d %s %d", rfcommcmd, devicenum, baddr, channel);
	if (verbose)
		printf("%s\n", buf);
        system(buf);
        exit(0);
    }

    if (argv[2] == NULL ||
        strncasecmp(argv[2], "-noppp", strlen(argv[2])) != 0) {
        sleep(5);
        runPPPD(configname, verbose);
    }
}


int getDevices(int display, char *findstr) {

    FILE *fs = NULL;
    DIR *bdir;
    struct dirent *de;
    struct stat fst;
    char fname[2048];
    char dirname[2048] = { 0 };
    time_t lasttime = 0;
    int  nodirs = 0;
    char cmd[2048];
    int  found = 0;

    /* Look at bluetooth cache directory first
     */
    bdir = opendir(bluetoothdir);

    if (bdir != NULL) {
        do {
            de = readdir(bdir);
            if (de != NULL) {
                if (strcmp(de->d_name, ".") != 0 &&
                        strcmp(de->d_name, "..") != 0) {
                    strcpy(fname, bluetoothdir);
                    strcat(fname, "/");
                    strcat(fname, de->d_name);
                    stat(fname, &fst);
                    if (lasttime < fst.st_mtime && de->d_type == DT_DIR) {
                        strcpy(dirname, fname);
                        lasttime = fst.st_mtime;
                    }
                }
            }
        }
        while (de != NULL);
        closedir(bdir);
        if (dirname[0] != '\0') {
            strcpy(fname, dirname);
            strcat(fname, "/names");
            fs = fopen(fname, "r");
        } else {
            nodirs = 1;
        }
    }


    if (display)
        printf("Devices currently found in cache:\n");

    if (fs != NULL) {
        found = lookupName(fs, display, findstr, baddr, dname);
        if (!found) {
            fclose(fs);
	    fs = NULL;
            if (nodirs) {
                found = get_hciscan_list(display, findstr, baddr, dname);
                if (!found) {
                    printf("bluemodem error: cannot find the requested device %s\n", findstr);
                }
                return found;
            }
            printf("Now starting to look for devices that are active!\n");
            found = get_hciscan_list(display, findstr, baddr, dname);
            if (!found) {
                if (!display) {
                    fclose(fs);
                    printf("bluemodem error: cannot find the requested device %s\n", findstr);
                    return 0;
                }
            }
        } else {
            found = get_hciscan_list(display, findstr, baddr, dname);
        }
    }

    if (fs != NULL)
        fclose(fs);

    if (display) {
        printf("Please select a device from the above list\n");
    }
    return found;
}


int lookupName(FILE *fs, int display, char *findstr, char *baddr, char *dname)
{
	char buf[8192];
	char dbaddr[8192];
	char devname[8192];
	int  found = 0;

	fgets(buf, sizeof(buf), fs);
	while (!feof(fs))
	{
		sscanf(buf, "%s %s", dbaddr, devname);
		if (!display)
		{
			/* check if the argument matches a device name
			 */
			if (strncasecmp(findstr, devname, strlen(findstr)) == 0)
			{
				strcpy(baddr, dbaddr);
				strcpy(dname, devname);
				found = 1;
			}
			/* Check if argument 1 is an address
			 */
			if (strncasecmp(findstr, dbaddr, strlen(findstr)) == 0)
			{
				strcpy(baddr, dbaddr);
				strcpy(dname, devname);
				found = 1;
			}
		}
		buf[0] = 0;
		fgets(buf, sizeof(buf), fs);
		if (display)
			printf("%s  %s\n", dbaddr, devname);
	}
	return found;
}

/* Check to see if the bluetooth configuration files exist
 */
int testBluetoothSetup(char *cfgname)
{
	FILE *fh = NULL;
	char filename[8192];

	strcpy(filename, dirppppeers);
	strcat(filename, cfgname);
	strcat(filename, ".bm");
	fh = fopen(filename, "r");
	if (fh == NULL)
		return 1;
	fclose(fh);

	strcpy(filename, dirppppeers);
	strcat(filename, cfgname);
	fh = fopen(filename, "r");
	if (fh == NULL)
		return 2;
	fclose(fh);
	return 0;
}

// Prompt the user for the configuration data
int promptSettings()
{
	char tmp[256];
	int  found = 1;

	while(found)
	{
		printf("\nEnter a configuration name (%s)\n", configname);
		scanf("%60s", tmp);
		tmp[60] = 0;
		found = getConfiguration(tmp);
		if (found)
			printf("\nThat configuration name already exists!\n");
	}
	strcpy(configname, tmp);
	dropSpecialChars(configname);

	while(!found)
	{
		getDevices(1, NULL);
		printf("\nPlease enter in a device name or address\n");
		scanf("%60s", tmp);
		dropSpecialChars(tmp);
		found = getDevices(0, tmp);
	}

	printf("\nEnter the phone number to access the internet.  Phone Providers\nwill usually give you an access # like the following:\n*99#\n*99***1#\n*99***2#\n*99**1#\n#777\n#2#\nCheck with you phone provider if you are unsure.\nEnter the phone # (access #) :");

	scanf("%20s", tmp);
	strncpy(phonenum, tmp, 20);
	phonenum[20] = 0;
	dropSpecialChars(phonenum);

	printf("\nEnter a the device name or type default (/dev/rfcomm0)\n");
	
	scanf("%60s", tmp);
	if (strcasecmp(tmp, "default") != 0)
	{
		strncpy(commdevice, tmp, 60);
		commdevice[60] = 0;
	}
	dropSpecialChars(commdevice);

	printf("\nEnter a connection timeout value in seconds 10 - 60 (default 25)\n");
	scanf("%20s", tmp);
	timeout = atoi(tmp);

	printf("\nEnter a baud rate (default 115200)\nBaud rates are 9600, 14400, 19200, 38400, 57600, 115200, 230400\n");
	scanf("%20s", tmp);
	baudrate = atol(tmp);

	return saveFiles(configname);
}

// Save the bluetooth configuration files
int saveFiles(char *cfgname)
{
	FILE *fh;
	char buff[16384];
	char tmp[256];
	char *p;
	char filename[8192];

	if (timeout < 10)
		timeout = 25;
	if (timeout > 60)
		timeout = 60;

	if (baudrate == 0)
		baudrate = 115200;

	if (baudrate < 12000)
		baudrate = 9600;
	if (baudrate >= 12000 && baudrate < 16800)
		baudrate = 14400;
	if (baudrate >= 16800 && baudrate < 28800)
		baudrate = 19200;
	if (baudrate >= 28800 && baudrate < 48000)
		baudrate = 38400;
	if (baudrate >= 48000 && baudrate < 86400)
		baudrate = 57600;
	if (baudrate >= 86400 && baudrate < 172800)
		baudrate = 115200;
	if (baudrate >= 172800)
		baudrate = 230400;

	devicenum = getDeviceNumber();

	strcpy(buff, bluemodem);
	p = strstr(buff, phonenumstr);
	if (p != NULL)
	{
		*p = 0;
		strcat(buff, phonenum);
		strcat(buff, p+strlen(phonenumstr));
	}
	p = strstr(buff, timeoutstr);
	if (p != NULL)
	{
		sprintf(tmp, "%d", timeout);
		*p = 0;
		strcat(buff, tmp);
		p = strstr(bluemodem, timeoutstr);
		strcat(buff, p+strlen(timeoutstr));
	}

	/* Save the chat script
	 */
	strcpy(filename, dirppppeers);
	strcat(filename, cfgname);
	strcat(filename, ".bm");
	fh = fopen(filename, "w");
	if (fh != NULL)
	{
		fwrite(buff, 1, strlen(buff), fh);
		fclose(fh);
	}
	else
		return 1;

	strcpy(buff, peerblue);
	p = strstr(buff, commdevicestr);
	if (p != NULL)
	{
		*p = 0;
		strcat(buff, commdevice);
		p = strstr(peerblue, commdevicestr);
		strcat(buff, p+strlen(commdevicestr));
	}
	p = strstr(buff, baudratestr);
	if (p != NULL)
	{
		sprintf(tmp, "%d", baudrate);
		*p = 0;
		strcat(buff, tmp);
		p = strstr(peerblue, baudratestr);
		strcat(buff, p+strlen(baudratestr));
	}
	p = strstr(buff, chatcmdstr);
	if (p != NULL)
	{
		*p = 0;
		strcat(buff, chatcmd);
		p = strstr(peerblue, chatcmdstr);
		strcat(buff, p+strlen(chatcmdstr));
	}
	p = strstr(buff, filebluemodemstr);
	if (p != NULL)
	{
		*p = 0;
		strcat(buff, filename);
		p = strstr(peerblue, filebluemodemstr);
		strcat(buff, p+strlen(filebluemodemstr));
	}

	/* Save the pppd configuration
	 */
	strcpy(filename, dirppppeers);
	strcat(filename, cfgname);
	fh = fopen(filename, "w");
	if (fh != NULL)
	{
		fwrite(buff, 1, strlen(buff), fh);
		fclose(fh);
	}
	else
		return 1;

	/* Save the new configuration entry
	 */
	fh = fopen(sysconfigname, "a");
	if (fh != NULL)
	{
		fprintf(fh, "%s,%s,%s\n", configname, dname, baddr);
		fclose(fh);
	}

	return 0;
}

/* Drop the pppd connection for the bluetooth
 */
int dropConnection(char *cfgname)
{
	int   pid;

	pid = lookPPPDConnection(cfgname);

	// kill the pppd daemon if found

	if (pid > -1)
	{
		kill(pid, 1);
		sleep(4);
		return 1;
	}

	return 0;
}

/* Look for the pppd connection */
int lookPPPDConnection(char *cfgname)
{
	FILE *fs;
	char  buf[8192];
	char  lookstr[8192];
	int   foundconn = 0;
	int   pid = -1;

	dropSpecialChars(cfgname);

	sprintf(buf, "%s axf", pscmd);
	fs = popen(buf, "r");
	if (fs != NULL)
	{
		sprintf(lookstr, "%s call %s", pppdcmd, cfgname);
		while (!feof(fs) && foundconn == 0)
		{
			fgets(buf, sizeof(buf), fs);
			if (strstr(buf, lookstr) != NULL)
			{
				pid = atoi(buf);
				foundconn = 1;
			}
		}
		fclose(fs);
	}

	return pid;
}


// Lookup the channel of the bluetooth device
int getChannel()
{
	FILE *fs;
	int  chl = -1;
	char buff[8192];
	char *p;
	int  founddial = 0;

	dropSpecialChars(baddr);

	fixbaddr(baddr);

	chl = get_Channel(baddr);

	return chl;
}


/* Run connect to the cell phone modem and update the dns
 */
int runPPPD(char *cfgname, int verbose)
{
	FILE *fs;
	int  chl = -1;
	char buf[8192];
	char lookstr[8192];
	char *p;
	int  foundrfcomm = 0;
	int  rpid = -1;
	int  i;
	int  pid;

	dropSpecialChars(cfgname);

	pid = fork();
	if (pid == 0)
	{
		/* update the dns in /etc/resolv.conf
		 */
		checkToUpdateResolv(cfgname);
		exit(0);
	}

	sprintf(lookstr, "%s connect %d", rfcommcmd, devicenum);

	/* Loop for 1 minute 15 times x delay 3 seconds and another delay of 2 seconds
	 */
	for (i=0; i<15; i++)
	{
		foundrfcomm = 0;
		sprintf(buf, "%s axf", pscmd);
		fs = popen(buf, "r");
		if (fs != NULL)
		{
			while (!feof(fs))
			{
				fgets(buf, sizeof(buf), fs);
				if (strstr(buf, lookstr) != NULL)
				{
					rpid = atoi(buf);
					foundrfcomm = 1;
				}
			}
			fclose(fs);
		}

		/* Start the pppd daemon if it is not started
		 * and connect to the bluetooth device
		 */
		if (foundrfcomm)
		{
			if (!checkPPPD(cfgname))
			{
				sprintf(buf, "%s call %s 2> /dev/null > /dev/null", pppdcmd, cfgname);
				if (verbose)
					printf("%s\n", buf);
				system(buf);
				printf("Connected to %s - %s\n", cfgname, dname);
			}
			else
				return 0;
		}
		else
			return 1;

		sleep(3);

	}

	return 1;
}

/* Check if pppd with bluemodem is already running
 */
int checkPPPD(char *cfgname)
{
	FILE *fs;
	int  chl = -1;
	char buf[8192];
	char tmp[8192];
	char lookstr[8192];
	char *p;
	int  foundpppd = 0;
	int  ppid = -1;

	sprintf(buf, "%s axf", pscmd);
	fs = popen(buf, "r");
	if (fs != NULL)
	{
		sprintf(lookstr, "%s call %s", pppdcmd, cfgname);
		while (!feof(fs))
		{
			fgets(buf, sizeof(buf), fs);
			if (strstr(buf, lookstr) != NULL)
			{
				ppid = atoi(buf);
				foundpppd = 1;
			}
		}
		fclose(fs);
	}

	if (foundpppd)
	{
		return 1;
	}

	sleep(2);

	return 0;
}

/* Check if the pppd resolv.conf file exists
 * if it does then get all the nameservers
 */
int checkPPPResolv()
{
	FILE *fs;
	int  chl = -1;
	char buf[8192];
	char look[8192];
	char *p;
	char ns[8192];
	char name[8192];
	char pppdid[8192];
	char ipaddr[8192];
	int  foundpppd = 0;
	int  pid;

	nscnt = 0;

	fs = fopen(newfilepppresolv, "r");
	if (fs != NULL)
	{
		while (!feof(fs) && nscnt < 2)
		{
			fgets(buf, sizeof(buf), fs);
			sscanf(buf, "%s %s", ns, name);
			if (strstr(ns, "nameserver") != NULL)
			{
				foundpppd = 1;
				strncpy(nameservers[nscnt], name, 30);
				nameservers[nscnt][30] = 0;
				nscnt++;
			}
		}
		fclose(fs);
	}

	// Some distros use the /etc/ppp/resolv.conf file
	if (!foundpppd)
	{
		fs = fopen(oldfilepppresolv, "r");
		if (fs != NULL)
		{
			while (!feof(fs) && nscnt < 2)
			{
				fgets(buf, sizeof(buf), fs);
				sscanf(buf, "%s %s", ns, name);
				if (strstr(ns, "nameserver") != NULL)
				{
					foundpppd = 1;
					strncpy(nameservers[nscnt], name, 30);
					nameservers[nscnt][30] = 0;
					nscnt++;
				}
			}
			fclose(fs);
		}
	}

	if (foundpppd)
		return 1;

	sleep(2);

	return 0;
}

/* Update the system resolv.conf file when pppd connects
 */
int checkToUpdateResolv(char *cfgname)
{
	int tries = 0;
	int ttimes = 0;

	sleep(1);
	while(1)
	{
		if (checkPPPResolv())
		{
			tries++;
			if (tries > 3)
			{
				updateResolv();
				break;
			}
		}
		else
			tries = 0;
		if (ttimes > 25)
			break;
		ttimes++;
	}

	return 0;
}



int updateResolv()
{
	FILE *fh;
	char buf[16384];
	char *wbuf;
	char nser[16384];
	char addr[16384];
	char tmp[16384];
	int  i,firstns = 1;
	int  savens = 0;
	int  sitem = 0;
	long fsize = 0;

	// Open the system resolv.conf file
	fh = fopen(filesysresolv, "r");

	// Get the file size
	fseek(fh, 0L, SEEK_END);
	fsize = ftell(fh);
	fseek(fh, 0L, SEEK_SET);
	// Allocate enough memory for the modified resolv.conf file
	wbuf = (char *) malloc(fsize + 1000);
	wbuf[0] = 0;

	if (fh != NULL)
	{
		while(!feof(fh))
		{
			nser[0] = 0;
			addr[0] = 0;
			buf[0] = 0;
			fgets(buf, sizeof(buf), fh);
			sscanf(buf, "%s %s", nser, addr);
			if (strncmp(nser, "nameserver", 10) == 0)
			{
				if (firstns)
				{
					if (strcmp(addr, nameservers[0]) != 0)
					{
						for (i=0; i<nscnt; i++)
						{
							sprintf(tmp, "nameserver %s\n", nameservers[i]);
							strcat(wbuf, tmp);
						}
						savens = 1;
					}
				}
				strcat(wbuf, buf);
				firstns = 0;
			}
			else
			{
				strcat(wbuf, buf);
			}
		}
		fclose(fh);
		if (firstns)
		{
			for (i=0; i<nscnt; i++)
			{
				sprintf(tmp, "nameserver %s\n", nameservers[i]);
				strcat(wbuf, tmp);
			}
			savens = 1;
		}
	}

	if (savens)
	{
		fh = fopen(filesysresolv, "w");
		if (fh != NULL)
		{
			fwrite(wbuf, 1, strlen(wbuf), fh);
			fclose(fh);
		}
	}
	free(wbuf);
}


int removeUpdatedResolv(char *cfgname)
{
	FILE *fh;
	char buf[16384];
	char wbuf[16384];
	char nser[16384];
	char addr[16384];
	char tmp[16384];
	int  i,firstns = 1;
	int  restorens = 0;
	int  savens = 0;
	int  sitem = 0;

	if (!checkPPPResolv())
		return 0;

	wbuf[0] = 0;
	fh = fopen(filesysresolv, "r");
	if (fh != NULL)
	{
		while(!feof(fh))
		{
			nser[0] = 0;
			addr[0] = 0;
			buf[0] = 0;
			fgets(buf, sizeof(buf), fh);
			sscanf(buf, "%s %s", nser, addr);
			if (strncmp(nser, "nameserver", 10) == 0)
			{
				if (firstns)
				{
					restorens = 0;
					for (i=0; i<nscnt; i++)
					{
						if (strcmp(addr, nameservers[i]) == 0)
						{
							restorens = 1;
							savens = 1;
						}
					}
					if (!restorens)
						strcat(wbuf, buf);
				}
				else
				{
					firstns = 0;
					strcat(wbuf, buf);
				}
			}
			else
			{
				strcat(wbuf, buf);
			}
		}
		fclose(fh);
	}

	if (savens)
	{
		fh = fopen(filesysresolv, "w");
		if (fh != NULL)
		{
			fwrite(wbuf, 1, strlen(wbuf), fh);
			fclose(fh);
		}
	}
}


/* Get the device from the peers pppd configuration
 * ----- This needs to be fixed ------
 */
int getDeviceFromFile(char *cfgname)
{
	FILE *fh;
	char filename[8192];
	char buf[16384];
	int  looknext = 0;
	int  found = 0;

	strcpy(filename, dirppppeers);
	strcat(filename, cfgname);
	fh = fopen(filename, "r");
	if (fh != NULL)
	{
		while(!feof(fh))
		{
			fgets(buf, sizeof(buf), fh);
			if (looknext)
			{
				sscanf(buf, "%s", commdevice);
				dropSpecialChars(commdevice);
				found = 1;
			}
			looknext = 0;
			if (strncmp(buf, "usepeerdns", 10) == 0)
				looknext = 1;
		}
		fclose(fh);
	}
	return found;
}

/* Get the device number from the name
 */

int getDeviceNumber()
{
	int num = 0;
	int i;

	for (i = strlen(commdevice); i>0; i--)
	{
		if (strcspn(commdevice+i, "0123456789") != 0)
		{
			i++;
			break;
		}
	}

	if (i == 0)
		return -1;
	return atoi(commdevice+i);
}


int listConfigurations()
{
	FILE *fh;
	char buf[8192];
	char *val;
	int  l;

	printf("%-25s %-25s %-25s\n", "Config Name", "Device Name", "Address");
	printf("%-25s %-25s %-25s\n", "-----------", "-----------", "-------");
	fh = fopen(sysconfigname, "r");
	if (fh != NULL)
	{
		while(!feof(fh))
		{
			buf[0] = 0;
			fgets(buf, sizeof(buf), fh);
			val = strtok(buf, ",");
			if (val != NULL)
				printf("%-25s ", val);
			val = strtok(NULL, ",");
			if (val != NULL)
				printf("%-25s ", val);
			val = strtok(NULL, ",");
			if (val != NULL)
				printf("%-25s\r", val);
		}
		fclose(fh);
	}
}


int getConfiguration(char *cfgname)
{
	FILE *fh;
	char buf[8192];
	char *val;
	int  found = 0,i,error;
	char tmpcfg[255];

	strcpy(tmpcfg, cfgname);
	fh = fopen(sysconfigname, "r");
	if (fh != NULL)
	{
		while(!feof(fh))
		{
			error = 0;
			buf[0] = 0;
			fgets(buf, sizeof(buf), fh);
			i = strlen(buf);
			if (i > 0)
				buf[i-1] = 0;
			val = strtok(buf, ",");
			if (val != NULL)
				strcpy(configname, val);
			val = strtok(NULL, ",");
			if (val != NULL)
				strcpy(dname, val);
			val = strtok(NULL, ",");
			if (val != NULL) {
				strcpy(baddr, val);
                        }
			else
				error = 1;
			if (error == 0 && strcmp(configname, tmpcfg) == 0)
			{
				found = 1;
				break;
			}
		}
		fclose(fh);
	}
	return found;
}


int removeConfiguration(char *cfgname)
{
	FILE *fh;
	char buf[2048];
	char tmp[2048];
	char wbuf[16384];
	char *p;
	int  updatecfg = 0;

	dropSpecialChars(cfgname);

	wbuf[0] = 0;
	fh = fopen(sysconfigname, "r");
	if (fh != NULL)
	{
		while(!feof(fh))
		{
			buf[0] = 0;
			tmp[0] = 0;
			fgets(buf, sizeof(buf), fh);
			strncpy(tmp, buf, 2048);
			p = strchr(tmp, ',');
			if (p != NULL)
			{
				*p = 0;
				if (strstr(tmp, cfgname) == NULL)
					strcat(wbuf, buf);
				else
					updatecfg = 1;
			}
		}
		fclose(fh);
	}

	if (updatecfg)
	{
		fh = fopen(sysconfigname, "w");
		if (fh != NULL)
		{
			fwrite(wbuf, 1, strlen(wbuf), fh);
			fclose(fh);
		}

		strcpy(buf, dirppppeers);
		strcat(buf, cfgname);
		unlink(buf);
		strcat(buf, ".bm");
		unlink(buf);
	}
	return updatecfg;
}

/* I used this function because of security
 * reasons.  When adding to the configuration files.
 */
void dropSpecialChars(char *data) {
    char *todata = data;
    while(*data != '\000') {
        if (*data == '\'' || *data == '\"' || *data == ';' ||
            *data == ':' || *data == '<' || *data == '>' ||
            *data == '\\' || *data == '!' || *data == ' ') {
            data++;
            continue;
        }
        *todata = *data;
        data++;
        todata++;
    }
    *todata = '\0';
}

